include "include/itemtypes";
include "../pkg/items/housing/housefriends";

Const HOUSE_LOCKDOWN_ITEM	:= 1;
Const HOUSE_RELEASE_ITEM		:= 2;
Const HOUSE_SECURE_CONTAINER	:= 3;
Const HOUSE_RELEASE_CONTAINER	:= 4;
Const HOUSE_RAISE_ITEM		:= 5;
Const HOUSE_LOWER_ITEM		:= 6;
Const HOUSE_DISPLAY_ITEM		:= 7;

///////////////////
//  gets the housevalue of the given account
///////////////////

function GetAccountHouseValue (acctname)
	var housevalue := GetGlobalProperty ("#housevalue of " + acctname);
	if (!housevalue)
		housevalue := 0;
	endif

	return housevalue;
endfunction




///////////////////
//  adds the value of the house to the owner's housevalue
///////////////////

function AddHouseValueToOwner (acctname, housevalue)
	set_critical (1);
	var oldhousevalue := GetAccountHouseValue (acctname);
	var newhousevalue := oldhousevalue + housevalue;
	SetGlobalProperty ("#housevalue of " + acctname, newhousevalue);
	set_critical (0);
endfunction




///////////////////
//  removes the value of the house from the owner's housevalue
///////////////////

function SubtractHouseValueFromOwner (acctname, housevalue)
	set_critical (1);
	var oldhousevalue := GetAccountHouseValue (acctname);
	var newhousevalue := oldhousevalue - housevalue;
	if (newhousevalue and newhousevalue > 0)
		SetGlobalProperty ("#housevalue of " + acctname, newhousevalue);
	else
		EraseGlobalProperty ("#housevalue of " + acctname);
	endif
	set_critical (0);
endfunction




///////////////////
//  determines if adding the housevalue will put a character over their limit
///////////////////

function HouseValueUnderLimit (character, housevalue);
	var oldhousevalue := GetAccountHouseValue (character.acctname);
	var newhousevalue := oldhousevalue + housevalue;
	if (newhousevalue < 6 )
		return 1;
	else
		return 0;
	endif
endfunction




///////////////////
//  Saves the location of unused items to a cfg file so they can be destroyed by a GM
///////////////////

function LogOldHouse (sign, ownerserial, MSG)

	set_critical (1);
	var prop := { };
	var thelog := array {};
	prop.+ pname := "House at " + sign.x + ", " + sign.y;
	prop.+ pvalue := "lastvisit: " + GetObjProperty (sign, "lastvisit");

	var elemkey := "owner serial: " + ownerserial;

	thelog[1] := prop;
	AppendConfigFileElem ( "housedecay", MSG, elemkey, thelog );
	UnloadConfigFile ("housedecay");
	set_critical (0);

endfunction




///////////////////
//  gets a default house price, based on its size
///////////////////

function GetDefaultHousePrice (housevalue)
	case (housevalue)
		2:
			return 90000;
		3:
			return 150000;
		4:
			return 400000;
		5:
			return 800000;
		default:
			return 40000;
	endcase
endfunction




///////////////////
//  determines the housevalue of a given deed
///////////////////

function GetHouseValueByDeed (deed)

	case (deed)
		0x6023: //tower
			return 3;
		0x6024: //keep
		0x6025: //castle
			return 5;
		0x6020: //largebrickhouse
		0x6021: //twostorywood
		0x6022: //twostorystone
		0x6026: //largepatiohousedeed
		0x6020: //largebrickhouse
		0x6021: //twostorywood
		0x6022: //twostorystone
		0x6229: //smalltowerdeed
		0x622A: //sandstonepatiohousedeed
		0x622B: //twostorylogcabindeed
		0x622C: //twostoryvilladeed
		0x622D: //largemarblepatiohousedeed
		0x622E: //ranch house
			return 2;
		default: return 1;
	endcase

endfunction




///////////////////
//  determines the housevalue of the given sign, whether its one of a static house or deedplaced
///////////////////

function GetHouseValueOfSign (sign)
	//static houses:
	if (sign.objtype == 0x7060)
		if ( GetObjProperty (sign, "housevalue") )
			return CINT (GetObjProperty (sign, "housevalue"));
		endif
		return 1;
	endif

	if ( GetObjProperty (sign, "housevalue") )
		return CINT (GetObjProperty (sign, "housevalue"));
	endif
	//deed-placed houses:
	var house := SystemFindObjectBySerial (GetObjProperty (sign, "house_serial"));
	var deed := GetDeedType (house);
	return GetHouseValueByDeed (deed);
endfunction




///////////////////
//  determines the deed objtype that corresponds to a given house type
///////////////////

function GetDeedType (house)

	var deed := 0;
	case (house.objtype)
		16484:	deed := 0x6019;
		16486:	deed := 0x601A;
		16490:	deed := 0x601B;
		16492:	deed := 0x601C;
		16494:	deed := 0x601D;
		16519:	deed := 0x6026;
		16500:	deed := 0x6020;
		16502:	deed := 0x6021;
		16504:	deed := 0x6022;
		16506:	deed := 0x6023;
		16508:	deed := 0x6024;
		16510:	deed := 0x6025;
		16524:	deed := 0x6026;
		16544:	deed := 0x6227;
		16546:	deed := 0x6228;
		16536:	deed := 0x6229;
		16540:	deed := 0x622A;
		16538:	deed := 0x622B;
		16542:	deed := 0x622C;
		16534:	deed := 0x622D;
		16525:	deed := 0x622E;
	endcase
	return deed;

endfunction




///////////////////
//  Determines the maximum number of lockdowns permitted in a house
///////////////////

function GetMaxLockdowns (sign)

	var hvalue := GetHouseValueOfSign (sign);
	return (hvalue * 30) + 10;

endfunction




///////////////////
//  returns the number of items locked down in the house
///////////////////

function GetLockdowns (sign)
	var itemlist := dictionary;
	foreach item in ListObjectsInBox (sign.x-32, sign.y-32, -128, sign.x+32, sign.y+32, +127)
		if (!item.movable)
			if ( GetObjProperty( item, "lockeddown" ) == sign.serial )
				if (!itemlist.exists(item) )
					itemlist[item] := item;
				endif
			endif
			if ( GetObjProperty( item, "gmlockeddown" ) == sign.serial )
				if (!itemlist.exists(item) )
					itemlist[item] := item;
				endif
			endif
		endif
	endforeach

	Setobjproperty (sign, "lockdowns", itemlist.size() );
	return itemlist.size();
endfunction




///////////////////
//  determines the maximum allowed number of secures for the house
///////////////////

function GetMaxSecures( sign )
	var hvalue := GetHouseValueOfSign (sign);
	return (hvalue * 4);
endfunction
  

///////////////////
//  counts the number of secured items in the house
///////////////////

function GetSecures (sign)
	var itemlist := dictionary;
	foreach item in ListObjectsInBox (sign.x-32, sign.y-32, -128, sign.x+32, sign.y+32, +127)
		if (!item.movable)
			if ( GetObjProperty (item, "secured") == sign.serial )
				if (!itemlist.exists(item) )
					itemlist[item] := item;
				endif
			endif
		endif
	endforeach

	Setobjproperty (sign, "secures", itemlist.size() );
	return itemlist.size();
endfunction




///////////////////
//  refreshes the house
///////////////////

function RefreshHouse (sign)
	var lastvisit := GetObjProperty (sign, "lastvisit");
	if (!lastvisit)
		SetObjProperty (sign, "lastvisit", ReadGameClock() );
	elseif ( lastvisit < ReadGameClock() )
		SetObjProperty (sign, "lastvisit", ReadGameClock() );
	endif
endfunction




///////////////////
//  refreshes the house for a longer period of time
///////////////////

function LongTermRefresh (sign, owner)
	var onemonth := 2592000 + ReadGameClock();
	SetObjProperty (sign, "lastvisit" , onemonth);
	SendSysMessage (owner, "Your house has been refreshed for one month.");
endfunction




///////////////////
//  releases all locked down and secured items in the house.  Used when the house is destroyed
///////////////////

function ReleaseAll (owner, sign)
	var homeinfo := GetObjProperty (sign, "homeinfo");
	foreach item in ListObjectsInBox (sign.x-32, sign.y-32, -128, sign.x+32, sign.y+32, +127)
		if (!item.movable and item.isa (POLCLASS_ITEM))
			if ( (item.x >= homeinfo[2] and item.x <= homeinfo[4]) and
					(item.y >= homeinfo[3] and item.y <= homeinfo[5]) )
				if (!IsGMLockedDownItem (owner, item))
					var parms := {};
					parms[1] := sign;
					parms[2] := owner;
					parms[3] := item;
					parms[4] := 0;
					if (GetObjProperty (item, "displaycontainer") == sign.serial)
						parms[4] := HOUSE_DISPLAY_ITEM;
						run_script_to_completion (":housing:lockunlock", parms);
					elseif (GetObjProperty (item, "lockeddown") == sign.serial)
						parms[4] := HOUSE_LOCKDOWN_ITEM;
						run_script_to_completion (":housing:lockunlock", parms);
					elseif (GetObjProperty (item, "secured") == sign.serial)
						parms[4] := HOUSE_SECURE_CONTAINER;
						run_script_to_completion (":housing:lockunlock", parms);
					endif
				endif
			endif
		endif
	endforeach

	EraseObjProperty (sign, "lockdowns");
	EraseObjProperty (sign, "secures" );
endfunction




///////////////////
//  releases a locked down or secured item
///////////////////

function ReleaseLockedDownItem (owner, item)
	if (GetObjProperty (item, "olditem"))
		if (!ReserveItem (item))
			return;
		endif
		var oldtype := GetObjProperty (item, "olditem");
		var olditem := CreateItemAtLocation (item.x, item.y, item.z, oldtype, 1);
		olditem.name := item.name;
		olditem.color := item.color;
		olditem.graphic := item.graphic;
		olditem.movable := 1;
		if (getobjproperty (item, "lockid"))
			setobjproperty(olditem, "lockid", getobjproperty (item, "lockid") );
		endif
		PrinttextAbovePrivate (olditem, "Released!", owner);
		DestroyItem (item);
	else
		item.movable := 1;
		PrinttextAbovePrivate (item, "Released!", owner );
		ReleaseItem (item);
		if (item.decayat)
			item.decayat := ReadGameClock() + 600;
		endif
	endif
endfunction




///////////////////
//  allows the owner to pick a new owner for the house
///////////////////

function ChangeOwner (who, sign)
	SendSysMessage (who, "Transfer ownership to whom?");
	var newowner := Target (who);
	if (!newowner or !newowner.acctname)
		SendSysMessage (who, "Canceled.");
		return;
	endif

	if (!HouseValueUnderLimit (newowner, GetHouseValueOfSign (sign)))
		SendSysMessage (who, "That person owns too many houses!");
		return;
	endif

	AddHouseValueToOwner (newowner.acctname, GetHouseValueOfSign (sign));
	SubtractHouseValueFromOwner (who.acctname, GetHouseValueOfSign (sign));

	EraseObjProperty (sign, "housefriends");
	SetObjProperty (sign, "account", newowner.acctname);

	if (GetObjProperty (sign, "homeinfo"))
		var homeinfo := GetObjProperty (sign, "homeinfo");
		homeinfo[1] := newowner.serial;
		SetObjProperty (sign, "homeinfo", homeinfo);
	else
		var house := SystemFindObjectBySerial (GetObjProperty (sign, "house_serial"));
		SetObjProperty (house, "ownerserial", newowner.serial);
	endif

	SetObjProperty (sign, "lastownername", newowner.name);

	SendSysMessage (who, "You are no longer the owner of this house!");
	SendSysMessage (newowner, "You are now the owner of this house!");
	SendSysMessage (newowner, "You should change the locks soon.");
endfunction




///////////////////
//  Finds any GM lockeddown items in the house and refunds 1/2 the purchase price to the houseowner
///////////////////

function IsGMLockedDownItem (owner, item)
	if (!GetObjProperty (item, "gmlockeddown"))
		return 0;
	endif
	
	var price := GetObjProperty (item, "price");
	if (!price)
		DestroyItem (item);
		return 1;
	endif
	
	price := CINT (price/2);

	CreateItemInContainer (owner.backpack, UOBJ_GOLD_COIN, price);
	SendSysMessage (owner, "You recieve " + price + " gold back for " + item.desc);
	DestroyItem (item);
	return 1;
endfunction





///////////////////
//  does stuff when the house sign is created and on startup to make sure that the house is properly
//  credited to the right account, and decays old, unused houses
///////////////////

function SetAccount (sign, ownerserial)

	var char;
	var owneraccount := GetObjProperty (sign, "account");
	if (!owneraccount)
		foreach acctname in ListAccounts ()
			var acct := FindAccount(acctname);
			for i := 1 to 5
				char := acct.GetCharacter (i);
				if (char)
					if (char.serial == ownerserial)
						SetObjProperty (sign, "account", acct.name);
						return acct.name;
					endif
				endif
			endfor
		endforeach
		LogOldHouse (sign, ownerserial, "NO CHARACTER!");
	else
		var acct := FindAccount (owneraccount);
		if (!acct)
			LogOldHouse (sign, ownerserial, "NO ACCOUNT!");
		elseif (acct.banned) 
			LogOldHouse (sign, ownerserial, "ACCOUNT BANNED!");
		endif

		var lastvisit := GetObjProperty (sign, "lastvisit");
		if (!lastvisit)
			lastvisit := 0;
		endif

		var twoweeks := 1209600 * 2;
		if (lastvisit + twoweeks < ReadGameClock() )
			LogOldHouse (sign, ownerserial, "ACCOUNT: " + owneraccount + " UNVISITED SINCE: "+lastvisit );
		endif

		AddHouseValueToOwner (owneraccount, GetHouseValueOfSign (sign));

	endif
endfunction



///////////////////
//  handles speech events that the sign hears
///////////////////

function handle_speech (event, sign)
	RefreshHouse (sign);

	var text := lower( event.text );
	if (text["i wish to lock this down"] or text["i wish to release this"])
		set_critical (1);
		LockdownItem (sign, event.source);
		set_critical (0);
	elseif (text["i wish to secure this"] or text["i wish to unsecure this"])
		set_critical (1);
		SecureItem (sign, event.source);
		set_critical (0);
	elseif (text["i wish to display this"] or text["i wish to remove this from display"])
		set_critical (1);
		DisplayContainer (sign, event.source);
		set_critical (0);
	elseif (text["i wish to raise this"])
		set_critical (1);
		RaiseItem (sign, event.source);
		set_critical (0);
	elseif (text["i wish to lower this"])
		set_critical (1);
		LowerItem (sign, event.source);
		set_critical (0);
	elseif (text["i wish to refresh this house"])
		LongTermRefresh (sign, event.source);
	endif

endfunction




///////////////////
//  allows the character to lock down items
///////////////////

function LockdownItem (sign, owner)
	var parms := {};
	parms[1] := sign;
	parms[2] := owner;
	parms[3] := 0;
	parms[4] := HOUSE_LOCKDOWN_ITEM;
	start_script (":housing:lockunlock", parms);
endfunction




///////////////////
// allows the player to secure containers
///////////////////

function SecureItem (sign, owner)
	var parms := {};
	parms[1] := sign;
	parms[2] := owner;
	parms[3] := 0;
	parms[4] := HOUSE_SECURE_CONTAINER;
	start_script (":housing:lockunlock", parms);
endfunction




///////////////////
//  allows the house owner to raise the z position of items in the house
///////////////////

function RaiseItem (sign, owner)
	var parms := {};
	parms[1] := sign;
	parms[2] := owner;
	parms[3] := 0;
	parms[4] := HOUSE_RAISE_ITEM;
	start_script (":housing:lockunlock", parms);
endfunction




///////////////////
//  allows the house owner to lower the z position of items in the house
///////////////////

function LowerItem (sign, owner)
	var parms := {};
	parms[1] := sign;
	parms[2] := owner;
	parms[3] := 0;
	parms[4] := HOUSE_LOWER_ITEM;
	start_script (":housing:lockunlock", parms);

endfunction




///////////////////
//  Puts a container on display
///////////////////

function DisplayContainer (sign, owner)
	var parms := {};
	parms[1] := sign;
	parms[2] := owner;
	parms[3] := 0;
	parms[4] := HOUSE_DISPLAY_ITEM;
	start_script (":housing:lockunlock", parms);

endfunction



function IsInside (owner, sign, item)
	var homeinfo := GetObjProperty (sign, "homeinfo");
	if ( (item.x >= homeinfo[2] and item.x <= homeinfo[4]) and
	     (item.y >= homeinfo[3] and item.y <= homeinfo[5]) )
		return 1;
	else
		SendSysmessage (owner, "That is not inside your house." );
		return 0;
	endif
endfunction




///////////////////
//  tries to find a house sign near the character that belongs to that character
///////////////////

function FindHouseSign (character)
	foreach item in ListObjectsInBox (character.x-32, character.y-32, -128, character.x+32, character.y+32, +127)
		case (item.objtype)
			0x7060:
			0x0bd2:
			0x0bd0:
				if (IsAFriend (item, character) or character.cmdlevel)
					if (IsInsideTheHouse (character, item) )
						return item;
					endif
				endif
		endcase
	endforeach
	return 0;
endfunction




///////////////////
//  determines if the character is within the area the house covers
///////////////////

function IsInsideTheHouse (character, sign)
	var homeinfo := GetObjProperty (sign, "homeinfo");
	if (!homeinfo)
		return;
	endif

	if ( (character.x >= homeinfo[2] and character.x <= homeinfo[4]) and
	     (character.y >= homeinfo[3] and character.y <= homeinfo[5]) )
		return 1;
	else
		return 0;
	endif
endfunction




///////////////////
//  tries to find a house sign that covers the given location
///////////////////

function FindHouseSignAtLocation (x, y)
	foreach item in ListObjectsInBox (x-32, y-32, -128, x+32, y+32, +127)
		case (item.objtype)
			0x7060:
			0x0bd2:
			0x0bd0:
				if (IsLocationWithinHouseSign (item, x, y))
					return item;
				endif
		endcase
	endforeach
	return 0;
endfunction




///////////////////
//  determines if the given sign covers the given X-Y location
///////////////////

function IsLocationWithinHouseSign (sign, x, y)
	var homeinfo := GetObjProperty (sign, "homeinfo");
	if (!homeinfo)
		return 0;
	endif

	if ( (x >= homeinfo[2] and x <= homeinfo[4]) and
	     (y >= homeinfo[3] and y <= homeinfo[5]) )
		return 1;
	else
		return 0;
	endif
endfunction
